/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	December 01 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"
#include "resource/tb_open.xpm"
#include "resource/tb_save.xpm"
#include "resource/tb_add.xpm"
#include "resource/tb_edit.xpm"
#include "resource/tb_remove.xpm"
#include "resource/tb_make.xpm"
#include "resource/tb_clean.xpm"
#include "resource/tb_run.xpm"
#include "resource/tb_kill.xpm"
#include "resource/tb_settings.xpm"


/*=========================================================

	Variable Declarations

=========================================================*/
GtkWidget *main_win;
GdeWindowInfo *main_win_info;

GtkAccelGroup *main_accel_group;
static GtkTooltips *recent_tooltips;
static GtkItemFactoryEntry main_menu_items[] = {
	{ "/_File",						NULL,			NULL,						0, "<Branch>" },
	{ "/_File/Tear",				NULL,			NULL,						0, "<Tearoff>" },
	{ "/File/_Open Project...",		"<control>O",	project_open,				0, NULL },
	{ "/File/_Save Project",		"<control>S",	project_save,				0, NULL },
	{ "/File/Save Project _As...",	NULL,			project_save_as,			0, NULL },
	{ "/File/_Close Project",		"<control>W",	(void*) project_close,		0, NULL },
	{ "/File/Sep",					NULL,			NULL,						0, "<Separator>" },
	{ "/File/_Recent Projects",		NULL,			NULL,						0, "<Branch>" },
	{ "/File/Sep",					NULL,			NULL,						0, "<Separator>" },
	{ "/File/E_xit",				"<control>X",	(void*) main_win_destroy,	0, NULL },
	{ "/_View",						NULL,			NULL,						0, "<Branch>" },
	{ "/_View/Tear",				NULL,			NULL,						0, "<Tearoff>" },
	{ "/View/_Project Explorer",	"<control>P",	proj_win_show,				0, NULL },
	{ "/View/_Messages Window",		"<control>M",	status_win_show,			0, NULL },
	{ "/View/Sep",					NULL,			NULL,						0, "<Separator>" },
	{ "/View/_Clear Messages",		NULL,			status_win_clear,			0, NULL },
	{ "/View/Sep",					NULL,			NULL,						0, "<Separator>" },
	{ "/View/P_references...",		NULL,			settings_win_preferences, 	0, NULL },
	{ "/_Project",					NULL,			NULL,						0, "<Branch>" },
	{ "/_Project/Tear",				NULL,			NULL,						0, "<Tearoff>" },
	{ "/Project/_Add File(s)...",	"<control>A",	project_add,				0, NULL },
	{ "/Project/_Edit File...",		"<control>E",	project_edit,				0, NULL },
	{ "/Project/Re_move File",		"<control>R",	project_remove,				0, NULL },
	{ "/Project/Sep",				NULL,			NULL,						0, "<Separator>" },
	{ "/Project/_Settings...",		"F10",			settings_win_project,		0, NULL },
	{ "/_Command",					NULL,			NULL,						0, "<Branch>" },
	{ "/_Command/Tear",				NULL,			NULL,						0, "<Tearoff>" },
	{ "/Command/_Make",				"F7",			project_make,				0, NULL },
	{ "/Command/_Run",				"F5",			project_run,				0, NULL },
	{ "/Command/_Clean",			"F6",			project_clean,				0, NULL },
	{ "/Command/Sep",				NULL,			NULL,						0, "<Separator>" },
	{ "/Command/_Kill",				"F12",			project_kill_command,		0, NULL },
	{ "/_Help",						NULL,			NULL,						0, "<LastBranch>" },
	{ "/Help/_Manual...",			NULL,			NULL,						0, NULL },
	{ "/Help/_About...",			NULL,			about_win,					0, NULL }
};


/*---------------------------------------------------------

	main_win_menubar_create()

	Creates the main menu in the main window.

---------------------------------------------------------*/
static GtkWidget *main_win_menubar_create(void)
{
	GtkWidget		*menubar;
	GtkItemFactory	*main_menu_factory;

	gint nmenu_items = sizeof(main_menu_items)/sizeof(main_menu_items[0]);
	main_accel_group = gtk_accel_group_new();

	main_menu_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", main_accel_group);
	gtk_item_factory_create_items(main_menu_factory, nmenu_items, main_menu_items, NULL);
	gtk_object_set_data(GTK_OBJECT (main_win), "main_menu_factory", main_menu_factory);
	
	menubar = gtk_item_factory_get_widget(main_menu_factory, "<main>");
	gtk_window_add_accel_group(GTK_WINDOW (main_win), main_accel_group);
	return menubar;
}


/*---------------------------------------------------------

	xpm_to_widget()

	Convert an included xpm to a widget.

---------------------------------------------------------*/
static GtkWidget* xpm_to_widget(gchar **xpm)
{
	GdkPixmap *pixmap;
	GdkBitmap *mask;

	pixmap = gdk_pixmap_create_from_xpm_d(main_win->window, &mask, NULL, xpm);
	return gtk_pixmap_new(pixmap, mask);
}


/*---------------------------------------------------------

	main_win_toolbar_create()

	Creates the toolbar of the main win.

---------------------------------------------------------*/
GtkWidget *main_win_toolbar_create(void)
{
	GtkWidget *toolbar;
	GtkWidget *item;

	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_button_relief(GTK_TOOLBAR (toolbar), GTK_RELIEF_NONE);
	gtk_toolbar_set_space_style(GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
	gtk_container_set_border_width(GTK_CONTAINER (toolbar), 1);

	gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Open", "Open project", NULL, 
							xpm_to_widget(tb_open_xpm), project_open, NULL);
	gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Save", "Save current project", NULL, 
							xpm_to_widget(tb_save_xpm), project_save, NULL);

	gtk_toolbar_append_space(GTK_TOOLBAR (toolbar));

	gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Add", "Add file(s) to project", NULL, 
							xpm_to_widget(tb_add_xpm), project_add, NULL);
	gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Edit", "Edit selected file", NULL, 
							xpm_to_widget(tb_edit_xpm), project_edit, NULL);
	gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Remove", "Remove selected file from project", NULL, 
							xpm_to_widget(tb_remove_xpm), project_remove, NULL);

	gtk_toolbar_append_space(GTK_TOOLBAR (toolbar));

	item = gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Make", "Make current project", NULL, 
							xpm_to_widget(tb_make_xpm), project_make, NULL);
	gtk_object_set_data(GTK_OBJECT (main_win), "toolbar_make_command", item);
	
	item = gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Edit", "Clean current project", NULL, 
							xpm_to_widget(tb_clean_xpm), project_clean, NULL);
	gtk_object_set_data(GTK_OBJECT (main_win), "toolbar_clean_command", item);
	
	item = gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Run", "Run current project", NULL, 
							xpm_to_widget(tb_run_xpm), project_run, NULL);
	gtk_object_set_data(GTK_OBJECT (main_win), "toolbar_run_command", item);

	gtk_toolbar_append_space(GTK_TOOLBAR (toolbar));

	item = gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Kill", "Kill current running command", NULL, 
							xpm_to_widget(tb_kill_xpm), project_kill_command, NULL);
	gtk_object_set_data(GTK_OBJECT (main_win), "toolbar_kill_command", item);

	gtk_toolbar_append_space(GTK_TOOLBAR (toolbar));

	gtk_toolbar_append_item(GTK_TOOLBAR (toolbar), "Settings", "Change settings", NULL, 
							xpm_to_widget(tb_settings_xpm), settings_win_project, NULL);

	return toolbar;
}


/*---------------------------------------------------------

	main_win_show()

	Pop the window on the screen.

---------------------------------------------------------*/
void main_win_show(void)
{
	if (!GTK_WIDGET_VISIBLE(main_win)) {
		main_win_set_window_info(main_win_info);
	}
}


/*---------------------------------------------------------

	main_win_hide()

	Remove the window from the screen.

---------------------------------------------------------*/
gint main_win_hide(void)
{
	if (GTK_WIDGET_VISIBLE(main_win)) {
		main_win_get_window_info(main_win_info);
		gtk_widget_hide(main_win);
	}
	return TRUE;
}


/*---------------------------------------------------------

	main_win_destroy

	Destroy the main window

---------------------------------------------------------*/
gint main_win_destroy(void)
{
	if (project_close()) {
		gde_write_rc();

		status_win_destroy();
		proj_win_destroy();
		project_delete();
	
		g_free(main_win_info);
		gtk_main_quit();

		return FALSE;
	}

	return TRUE;
}


/*---------------------------------------------------------

	main_win_create()

	Creates the main window, with GTK+.

---------------------------------------------------------*/
void main_win_create(void)
{
	GtkWidget *vbox;
	GtkWidget *menubar;
	GtkWidget *toolbar;
	
	main_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect(GTK_OBJECT (main_win), "delete_event",
					   GTK_SIGNAL_FUNC (main_win_destroy), NULL);
	gtk_window_set_policy(GTK_WINDOW (main_win), FALSE, TRUE, FALSE);
	gtk_widget_set_uposition(main_win, MAINWIN_LEFT, MAINWIN_TOP);
	gtk_widget_set_usize(main_win, MAINWIN_WIDTH, MAINWIN_HEIGHT);
	main_win_set_title(NULL);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER (main_win), vbox);
	gtk_widget_show(vbox);
	gtk_widget_realize(main_win);

	menubar = main_win_menubar_create();
	gtk_box_pack_start(GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
	gtk_widget_show(menubar);

	toolbar = main_win_toolbar_create();
	gtk_box_pack_start(GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
	gtk_widget_show(toolbar);

	main_win_info = g_malloc0(sizeof(GdeWindowInfo));
	gtk_widget_show(main_win);
}


/*---------------------------------------------------------

	main_win_set_title()

	Change the title of the main window with a filename

---------------------------------------------------------*/
void main_win_set_title(const char *filename)
{
	gchar title[255];

	if (filename) {
		if (project->needs_save)
			sprintf(title, "%s %s - *%s", MAIN_WIN_TITLE, GDE_VERSION, filename);
		else
			sprintf(title, "%s %s - %s", MAIN_WIN_TITLE, GDE_VERSION, filename);
	}
	else
		sprintf(title, "%s %s", MAIN_WIN_TITLE, GDE_VERSION);
		
	gtk_window_set_title(GTK_WINDOW (main_win), title);
}


/*----------------------------------------------------------

	main_win_make_command_sensitive()

	Changes sensitivity of make command.

----------------------------------------------------------*/
static void main_win_make_command_sensitive(gboolean s)
{
	GtkItemFactory *main_menu_factory;
	GtkWidget *widget;
	
	widget = (GtkWidget*) gtk_object_get_data(GTK_OBJECT (main_win), "toolbar_make_command");
	gtk_widget_set_sensitive(widget, s);

	main_menu_factory = gtk_object_get_data(GTK_OBJECT (main_win), "main_menu_factory");
	widget = gtk_item_factory_get_widget(main_menu_factory, "/Command/Make");
	gtk_widget_set_sensitive(widget, s);
}


/*----------------------------------------------------------

	main_win_clean_command_sensitive()

	Changes sensitivity of clean command.

----------------------------------------------------------*/
static void main_win_clean_command_sensitive(gboolean s)
{
	GtkItemFactory *main_menu_factory;
	GtkWidget *widget;
	
	widget = (GtkWidget*) gtk_object_get_data(GTK_OBJECT (main_win), "toolbar_clean_command");
	gtk_widget_set_sensitive(widget, s);

	main_menu_factory = gtk_object_get_data(GTK_OBJECT (main_win), "main_menu_factory");
	widget = gtk_item_factory_get_widget(main_menu_factory, "/Command/Clean");
	gtk_widget_set_sensitive(widget, s);
}


/*----------------------------------------------------------

	main_win_run_command_sensitive()

	Changes sensitivity of run command.

----------------------------------------------------------*/
static void main_win_run_command_sensitive(gboolean s)
{
	GtkItemFactory *main_menu_factory;
	GtkWidget *widget;
	
	widget = (GtkWidget*) gtk_object_get_data(GTK_OBJECT (main_win), "toolbar_run_command");
	gtk_widget_set_sensitive(widget, s);

	main_menu_factory = gtk_object_get_data(GTK_OBJECT (main_win), "main_menu_factory");
	widget = gtk_item_factory_get_widget(main_menu_factory, "/Command/Run");
	gtk_widget_set_sensitive(widget, s);
}


/*----------------------------------------------------------

	main_win_kill_command_sensitive()

	Changes sensitivity of kill command.

----------------------------------------------------------*/
static void main_win_kill_command_sensitive(gboolean s)
{
	GtkItemFactory *main_menu_factory;
	GtkWidget *widget;
	
	widget = (GtkWidget*) gtk_object_get_data(GTK_OBJECT (main_win), "toolbar_kill_command");
	gtk_widget_set_sensitive(widget, s);

	main_menu_factory = gtk_object_get_data(GTK_OBJECT (main_win), "main_menu_factory");
	widget = gtk_item_factory_get_widget(main_menu_factory, "/Command/Kill");
	gtk_widget_set_sensitive(widget, s);
}


/*----------------------------------------------------------

	main_win_disable_commands()

	Diables all commands; the tool and menubar items
	get grayed; unsensitive.

----------------------------------------------------------*/
void main_win_disable_commands(void)
{
	main_win_make_command_sensitive(FALSE);
	main_win_clean_command_sensitive(FALSE);
	main_win_run_command_sensitive(FALSE);
	main_win_kill_command_sensitive(TRUE);
}


/*----------------------------------------------------------

	main_win_enable_commands()

	Enable commands, counterpart of disable of course.

----------------------------------------------------------*/
void main_win_enable_commands(void)
{
	if (project->make_command[0])
		main_win_make_command_sensitive(TRUE);
	if (project->clean_command[0])
		main_win_clean_command_sensitive(TRUE);
	if (project->run_command[0])
		main_win_run_command_sensitive(TRUE);
	main_win_kill_command_sensitive(FALSE);
}


/*----------------------------------------------------------

	main_win_recent_open()

	This one opens a recent file list. This is the 
	callback of the itemfactory.

----------------------------------------------------------*/
static void main_win_recent_open(gpointer data)
{
	gchar* filename;

	if (project_close()) {
		filename = g_strdup(data); 	/* avoid strcpying in itself! */
		project_open_file(filename);/* goes way round back to add */ 
		g_free(filename);			/* took  an hour to fix this! */
	}
}


/*----------------------------------------------------------

	main_win_recent_clear()

	Clears the recent file list, not the one in memory
	but the one in the menu.

----------------------------------------------------------*/
void main_win_recent_clear(void)
{
	GtkItemFactory *main_menu_factory;
	GtkWidget *widget;
	
	gchar *menu_item_path;
	gchar strnum[10];
	gint i;

	main_menu_factory = gtk_object_get_data(GTK_OBJECT (main_win), "main_menu_factory");
	widget = gtk_item_factory_get_widget(main_menu_factory, "/File/Recent Projects");

	for (i = 0; i < gde_rc->recent_size; i++) {
		if (*gde_rc->recent_list[i]) {
			sprintf(strnum, "%i ", i+1);
			menu_item_path = g_strconcat("/File/Recent Projects/", strnum, g_basename(gde_rc->recent_list[i]), NULL);
			gtk_item_factory_delete_item(main_menu_factory, menu_item_path);
			g_free(menu_item_path);
		}
	}
	
	gtk_widget_set_sensitive(widget, FALSE);

	if (recent_tooltips) {
		gtk_object_destroy(GTK_OBJECT (recent_tooltips));
		gtk_object_unref(GTK_OBJECT (recent_tooltips));
		recent_tooltips = NULL;
	}
}


/*----------------------------------------------------------

	main_win_recent_update()

	Update the recent files list. Not the one in memory
	bu the one in the menu. This one puts all the items
	in the memory list onto the menu.

----------------------------------------------------------*/
void main_win_recent_update(void)
{
	GtkItemFactory *main_menu_factory;
	GtkItemFactoryEntry menu_item = { NULL, NULL, NULL, 0, NULL };
	GtkWidget *widget;
	
	gchar *menu_item_path;
	gchar strnum[10], menu_item_accel[12];
	gint i;

	main_menu_factory = gtk_object_get_data(GTK_OBJECT (main_win), "main_menu_factory");
	recent_tooltips = gtk_tooltips_new();
	
	for (i = 0; i < gde_rc->recent_size; i++) {
		if (*gde_rc->recent_list[i]) {
			sprintf(strnum, "_%i ", i+1);
			menu_item_path = g_strconcat("/File/Recent Projects/", strnum, g_basename(gde_rc->recent_list[i]), NULL);
			menu_item.path = menu_item_path;
			sprintf(menu_item_accel, "<control>%i", i+1);
			menu_item.accelerator = menu_item_accel;
			menu_item.callback = main_win_recent_open;
			gtk_item_factory_create_item(main_menu_factory, &menu_item, &gde_rc->recent_list[i], 1);
			g_free(menu_item_path);
			
			sprintf(strnum, "%i ", i+1);
			menu_item_path = g_strconcat("/File/Recent Projects/", strnum, g_basename(gde_rc->recent_list[i]), NULL);
			widget = gtk_item_factory_get_widget(main_menu_factory, menu_item_path);
			gtk_tooltips_set_tip(recent_tooltips, widget, gde_rc->recent_list[i], NULL);
			g_free(menu_item_path);
		}
	}
	
	widget = gtk_item_factory_get_widget(main_menu_factory, "/File/Recent Projects");
	gtk_widget_set_sensitive(widget, TRUE);
}


/*----------------------------------------------------------

	main_win_recent_add()

	Add a recent project to the recent projects list. This
	list contains a maximum of GDE_RECENT_MAX items, the
	actual number of items is saved in the gde_rc struct.
	It does not contain any doubles.

----------------------------------------------------------*/
void main_win_recent_add(const gchar *recent_file)
{
	gint i;

	if (recent_file) {
		main_win_recent_clear();

		for (i = 0; i < gde_rc->recent_size; i++) {
			if (!strcmp(gde_rc->recent_list[i], recent_file))
				break;
		}

		if (i != gde_rc->recent_size) {
			strcpy(gde_rc->recent_list[i], gde_rc->recent_list[0]);
			strcpy(gde_rc->recent_list[0], recent_file);
		}
		else {
			for (i = gde_rc->recent_size-2; i >= 0; i--) {
				strcpy(gde_rc->recent_list[i+1], gde_rc->recent_list[i]);
			}
		}

		strcpy(gde_rc->recent_list[0], recent_file);

		main_win_recent_update();
	}
}


/*----------------------------------------------------------

	main_win_get_window_info()

	Return the position, size and visibility state of 
	the window.

----------------------------------------------------------*/
void main_win_get_window_info(GdeWindowInfo *win_info)
{
	win_info->visible = GTK_WIDGET_VISIBLE(main_win);
	
	if (!win_info->visible) {
		win_info->x = main_win_info->x;	
		win_info->y = main_win_info->y;	
		win_info->width = main_win_info->width;	
		win_info->height = main_win_info->height;
	}
	else { 
		gdk_window_get_origin(GTK_WIDGET (main_win)->window, &win_info->x, &win_info->y);
		gdk_window_get_size(GTK_WIDGET (main_win)->window, &win_info->width, &win_info->height);
	}
}


/*----------------------------------------------------------

	main_win_set_window_info()

	Set the geometry and visibility of the window.

----------------------------------------------------------*/
void main_win_set_window_info(GdeWindowInfo *win_info)
{
	if (win_info->visible) {
		if (!GTK_WIDGET_VISIBLE(main_win))
			gtk_widget_show(main_win);

		gtk_widget_set_uposition(main_win, win_info->x, win_info->y);
		gdk_window_resize(main_win->window, win_info->width, win_info->height);
	}
	else {
		if (GTK_WIDGET_VISIBLE(main_win))
			main_win_hide();
		
		main_win_info->x = win_info->x;		/* Why this? I dunno,  seems like GTK has */
		main_win_info->y = win_info->y;		/* some bug in displaying  and hiding win */
		main_win_info->width = win_info->width;		/* Want to know why? Just comment */
		main_win_info->height = win_info->height; 	/* out these four  lines  of code */
		main_win_info->visible = TRUE;
	}
}
