/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	November 28 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"


/*=========================================================

	Variable Declarations

=========================================================*/
static GtkWidget *message_dialog;
static MessageWinButton button_pressed;


/*---------------------------------------------------------

	message_win_yes()

	This function is called when the user chooses yes.

---------------------------------------------------------*/
static void message_win_yes(GtkWidget *widget, gpointer data)
{
	button_pressed = MESSAGE_BUTTON_YES;

	gtk_widget_hide(GTK_WIDGET (message_dialog));
}


/*---------------------------------------------------------

	message_win_no()

	This function is called when the user chooses no.

---------------------------------------------------------*/
static void message_win_no(GtkWidget *widget, gpointer data)
{
	button_pressed = MESSAGE_BUTTON_NO;

	gtk_widget_hide(GTK_WIDGET (message_dialog));
}


/*---------------------------------------------------------

	message_win_cancel()

	This function is called when the user chooses cancel.

---------------------------------------------------------*/
static gint message_win_cancel(GtkWidget *widget, gpointer data)
{
	button_pressed = MESSAGE_BUTTON_CANCEL;

	gtk_widget_hide(GTK_WIDGET (message_dialog));
	return TRUE;
}


/*---------------------------------------------------------

	message_win()

	Shows a message dialog, with title as title and
	message as message. Hmm... Seems very logic to me!

---------------------------------------------------------*/
MessageWinButton message_win(const gchar *title, const gchar *message)
{
	GtkWidget *lab_message;
	GtkWidget *hbbox, *vbox;	
	GtkWidget *yes_button, *no_button, *cancel_button;
	
	message_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW (message_dialog), title);
	gtk_window_set_modal(GTK_WINDOW (message_dialog), TRUE);
	gtk_window_set_policy(GTK_WINDOW (message_dialog), FALSE, FALSE, FALSE);
	gtk_window_set_position(GTK_WINDOW (message_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW (message_dialog), GTK_WINDOW (main_win));
	gtk_signal_connect(GTK_OBJECT (message_dialog), "delete_event",
	 				   GTK_SIGNAL_FUNC(message_win_cancel), NULL);

	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);
	gtk_box_pack_start(GTK_BOX (GTK_DIALOG (message_dialog)->vbox), vbox, FALSE, FALSE, 0);
	gtk_widget_show(vbox);
	lab_message = gtk_label_new(message);
	gtk_box_pack_start(GTK_BOX (vbox), lab_message, FALSE, FALSE, 0);
	gtk_widget_show(lab_message);

	gtk_container_set_border_width(GTK_CONTAINER (GTK_DIALOG (message_dialog)->action_area), 2);
	gtk_box_set_homogeneous(GTK_BOX (GTK_DIALOG (message_dialog)->action_area), FALSE);
	
	hbbox = gtk_hbutton_box_new();
	gtk_button_box_set_spacing(GTK_BUTTON_BOX (hbbox), 4);
	gtk_box_pack_end(GTK_BOX (GTK_DIALOG (message_dialog)->action_area), hbbox, FALSE, FALSE, 0);
	gtk_widget_show(hbbox);

	yes_button = gtk_button_new_with_label("Yes");
	GTK_WIDGET_SET_FLAGS(yes_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT (yes_button), "clicked",
					   GTK_SIGNAL_FUNC(message_win_yes), NULL);
	gtk_box_pack_end(GTK_BOX (hbbox), yes_button, FALSE, FALSE, 0);
	gtk_widget_grab_default(yes_button);
	gtk_widget_show(yes_button);

	no_button = gtk_button_new_with_label("No");
	GTK_WIDGET_SET_FLAGS(no_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT (no_button), "clicked",
					   GTK_SIGNAL_FUNC(message_win_no), NULL);
	gtk_box_pack_end(GTK_BOX (hbbox), no_button, FALSE, FALSE, 0);
	gtk_widget_show(no_button);

	cancel_button = gtk_button_new_with_label("Cancel");
	GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT (cancel_button), "clicked",
					   GTK_SIGNAL_FUNC(message_win_cancel), NULL);
	gtk_box_pack_end(GTK_BOX (hbbox), cancel_button, FALSE, FALSE, 0);
	gtk_widget_show(cancel_button);

	/* Let the user do his things */
	gtk_widget_show(message_dialog);

	while (GTK_WIDGET_VISIBLE(message_dialog))
		gtk_main_iteration_do(TRUE);

	gtk_widget_destroy(message_dialog);
	return button_pressed;
}
