/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	November 28 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"


/*=========================================================

	Variable Declarations

=========================================================*/
static GdeExtType ExtTypeTable[EXT_TYPE_TABLE_LEN] = { 
	{ ".c",			's' }, { ".cpp",		's' },
	{ ".cxx",		's' }, { ".cc",			's' },
	{ ".htm",		's' }, { ".html",		's' },
	{ ".asp",		's' }, { ".php",		's' },
	{ ".php3",		's' }, { ".php4",		's' },
	{ ".tex",		's' }, { ".xml",		's' }, 
	{ ".pl",		's' }, { ".java",		's' },

	{ ".h",			'h' }, { ".hpp",		'h' },
	{ ".hxx",		'h' }, { ".hh",			'h' },
	{ ".css",		'h' }, { ".inc",		'h' },
	{ ".pm",		'h' }, { ".inc.php",	'h' },
	{ ".inc.php3",	'h' }, { ".inc.php4",	'h' }
};


/*---------------------------------------------------------

	type_from_filename()

	Returns a character representing the type of 
	filename. If it seems to be a source file, 's' is
	returned. If GDE thinks it is a header file 'h' is
	returned. Otherwise 'r' is returned.

---------------------------------------------------------*/
gchar type_from_filename(const gchar *filename)
{
	gchar buf[EXT_LEN];
	gint i;

	for (i = 0; i != EXT_TYPE_TABLE_LEN; i++) {
		memcpy(buf, filename+strlen(filename)-strlen(ExtTypeTable[i].ext), 
		       EXT_LEN);
		if (strstr(buf, ExtTypeTable[i].ext))
			return ExtTypeTable[i].type;
	}
	return 'r';
}


/*---------------------------------------------------------

	str_remove_substr()

	Remove the substring substr from the string str.

---------------------------------------------------------*/
gchar *str_remove_substr(gchar *str, const gchar *substr)
{
	gchar *ss, *end;
	ss = strstr(str, substr);

	if (ss) {
		end = ss + (sizeof(gchar) * strlen(substr));

		while (*end) {
			*ss = *end;
			ss++;
			end++;
		}
		*ss = '\0';
	}

	return str;
}


/*----------------------------------------------------------

	str_get_value_of()

	Read a resource value from a string. Checks for a
	given substring at the beginning, and removes it
	if it is there. After all this, removes remaining
	whitespace. Returns true if found, false otherwise.

----------------------------------------------------------*/
gboolean str_get_value_of(const gchar *str, gchar *buf)
{
	if (!strncmp(str, buf, strlen(str))) {
		str_remove_substr(buf, str);
		g_strstrip(buf);
		return TRUE;
	}
	return FALSE;
}
