/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	November 30 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"


/*=========================================================

	Variable Declarations

=========================================================*/
static GtkWidget	*settings_dialog;


/*---------------------------------------------------------

	settings_win_ok()

	This function is called when the user wants to accept
	his changes.

---------------------------------------------------------*/
static void settings_win_ok(GtkWidget *widget, gpointer data)
{
	GtkWidget *txt_edit_cmd, *txt_make_cmd, *txt_clean_cmd, *txt_run_cmd;
	GtkWidget *opt_check_dates;
	GtkWidget *txt_info_font, *txt_output_font, * txt_error_font;
	GtkWidget *spin_recent_size, *opt_save_window_info, *opt_open_last_opened;
	
	/* Project Settings */
	main_win_disable_commands();
	txt_edit_cmd = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "txt_edit_cmd");
	strcpy(project->edit_command, gtk_entry_get_text(GTK_ENTRY (txt_edit_cmd)));
	txt_make_cmd = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "txt_make_cmd");
	strcpy(project->make_command, gtk_entry_get_text(GTK_ENTRY (txt_make_cmd)));
	txt_clean_cmd = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "txt_clean_cmd");
	strcpy(project->clean_command, gtk_entry_get_text(GTK_ENTRY (txt_clean_cmd)));
	txt_run_cmd = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "txt_run_cmd");
	strcpy(project->run_command, gtk_entry_get_text(GTK_ENTRY (txt_run_cmd)));
	main_win_enable_commands();
	
	opt_check_dates = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "opt_check_dates");
	project->check_dates = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (opt_check_dates));

	project->needs_save = TRUE;
	main_win_set_title(g_basename(project->filename));

	/* GDE Preferences */
	txt_info_font = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "txt_info_font");
	strcpy(gde_rc->info_font, gtk_entry_get_text(GTK_ENTRY (txt_info_font)));
	txt_output_font = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "txt_output_font");
	strcpy(gde_rc->output_font, gtk_entry_get_text(GTK_ENTRY (txt_output_font)));
	txt_error_font = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "txt_error_font");
	strcpy(gde_rc->error_font, gtk_entry_get_text(GTK_ENTRY (txt_error_font)));
	
	main_win_recent_clear();
	spin_recent_size = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "spin_recent_size");
	gde_rc->recent_size = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (spin_recent_size));
	main_win_recent_update();

	opt_save_window_info = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "opt_save_window_info");
	gde_rc->save_window_info = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (opt_save_window_info));
	opt_open_last_opened = (GtkWidget *) gtk_object_get_data(GTK_OBJECT (settings_dialog), "opt_open_last_opened");
	gde_rc->open_last_opened = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (opt_open_last_opened));

	gtk_widget_hide(GTK_WIDGET (settings_dialog));
}


/*---------------------------------------------------------

	settings_win_cancel()

	This function is called whenever the user cancels
	changing project settings.

---------------------------------------------------------*/
static gint settings_win_cancel(GtkWidget *widget, gpointer data)
{
	gtk_widget_hide(GTK_WIDGET (settings_dialog));
	return TRUE;
}


/*---------------------------------------------------------

	settins_win()

	Shows the settings window. Pass along which tab
	number to show.

---------------------------------------------------------*/
static void settings_win(gint page)
{
	GtkWidget *table, *notebook, *hbbox, *vbox, *hbox, *frame;
	
	GtkWidget *lab_edit_cmd, *lab_make_cmd, *lab_clean_cmd, *lab_run_cmd;
	GtkWidget *txt_edit_cmd, *txt_make_cmd, *txt_clean_cmd, *txt_run_cmd;
	GtkWidget *opt_check_dates;
	
	GtkWidget *lab_info_font, *lab_output_font, *lab_error_font;
	GtkWidget *txt_info_font, *txt_output_font, *txt_error_font;
	GtkWidget *opt_save_window_info, *opt_open_last_opened;
	GtkWidget *lab_recent_size, *adj_recent_size, *spin_recent_size;
	
	GtkWidget *ok_button, *cancel_button;
	
	/* Create the settings window */
	settings_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW (settings_dialog), "Project Settings");
	gtk_window_set_modal(GTK_WINDOW (settings_dialog), TRUE);
	gtk_window_set_policy(GTK_WINDOW (settings_dialog), FALSE, FALSE, FALSE);
	gtk_window_set_position(GTK_WINDOW (settings_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW (settings_dialog), GTK_WINDOW (main_win));
	gtk_signal_connect(GTK_OBJECT (settings_dialog), "delete_event",
	 				   GTK_SIGNAL_FUNC(settings_win_cancel), NULL);

	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX (GTK_DIALOG (settings_dialog)->vbox), notebook, FALSE, FALSE, 0);
	gtk_widget_show(notebook);

	/* Create Project Settings tab */
	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);
	gtk_notebook_append_page(GTK_NOTEBOOK (notebook), vbox, gtk_label_new("Project Settings"));
	gtk_widget_show(vbox);
	
	frame = gtk_frame_new("Commands");
	gtk_box_pack_start(GTK_BOX (vbox), frame, TRUE, TRUE, 0);
	gtk_widget_show(frame);
	
	table = gtk_table_new(4, 4, TRUE);
	gtk_container_set_border_width(GTK_CONTAINER (table), 10);
	gtk_table_set_row_spacings(GTK_TABLE (table), 4);
	gtk_table_set_col_spacing(GTK_TABLE (table), 0, 20);
	gtk_container_add(GTK_CONTAINER (frame), table);
	gtk_widget_show(table);

	lab_edit_cmd = gtk_label_new("Edit Command: ");
	gtk_misc_set_alignment(GTK_MISC (lab_edit_cmd), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_edit_cmd), GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults(GTK_TABLE (table), lab_edit_cmd, 0, 1, 0, 1);
	gtk_widget_show(lab_edit_cmd);
	
	txt_edit_cmd = gtk_entry_new_with_max_length(255);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "txt_edit_cmd", txt_edit_cmd);
	gtk_entry_set_text(GTK_ENTRY (txt_edit_cmd), project->edit_command);
	gtk_table_attach_defaults(GTK_TABLE (table), txt_edit_cmd, 1, 4, 0, 1);
	gtk_widget_show(txt_edit_cmd);

	lab_make_cmd = gtk_label_new("Make Command: ");
	gtk_misc_set_alignment(GTK_MISC (lab_make_cmd), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_make_cmd), GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults(GTK_TABLE (table), lab_make_cmd, 0, 1, 1, 2);
	gtk_widget_show(lab_make_cmd);
	
	txt_make_cmd = gtk_entry_new_with_max_length(255);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "txt_make_cmd", txt_make_cmd);
	gtk_entry_set_text(GTK_ENTRY (txt_make_cmd), project->make_command);
	gtk_table_attach_defaults(GTK_TABLE (table), txt_make_cmd, 1, 4, 1, 2);
	gtk_widget_show(txt_make_cmd);

	lab_clean_cmd = gtk_label_new("Clean Command: ");
	gtk_misc_set_alignment(GTK_MISC (lab_clean_cmd), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_clean_cmd), GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults(GTK_TABLE (table), lab_clean_cmd, 0, 1, 2, 3);
	gtk_widget_show(lab_clean_cmd);

	txt_clean_cmd = gtk_entry_new_with_max_length(255);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "txt_clean_cmd", txt_clean_cmd);
	gtk_entry_set_text(GTK_ENTRY (txt_clean_cmd), project->clean_command);
	gtk_table_attach_defaults(GTK_TABLE (table), txt_clean_cmd, 1, 4, 2, 3);
	gtk_widget_show(txt_clean_cmd);

	lab_run_cmd = gtk_label_new("Run Command: ");
	gtk_misc_set_alignment(GTK_MISC (lab_run_cmd), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_run_cmd), GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults(GTK_TABLE (table), lab_run_cmd, 0, 1, 3, 4);
	gtk_widget_show(lab_run_cmd);
	
	txt_run_cmd = gtk_entry_new_with_max_length(255);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "txt_run_cmd", txt_run_cmd);
	gtk_entry_set_text(GTK_ENTRY (txt_run_cmd), project->run_command);
	gtk_table_attach_defaults(GTK_TABLE (table), txt_run_cmd, 1, 4, 3, 4);
	gtk_widget_show(txt_run_cmd);

	opt_check_dates = gtk_check_button_new_with_label("Check dates before run");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (opt_check_dates), project->check_dates);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "opt_check_dates", opt_check_dates);
	gtk_box_pack_start(GTK_BOX (vbox), opt_check_dates, TRUE, TRUE, 0);
	gtk_widget_show(opt_check_dates);

	/* GDE Preferences, Messages Window */
	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);
	gtk_notebook_append_page(GTK_NOTEBOOK (notebook), vbox, gtk_label_new("GDE Preferences"));
	gtk_widget_show(vbox);

	frame = gtk_frame_new("Messages Window");
	gtk_box_pack_start(GTK_BOX (vbox), frame, TRUE, TRUE, 0);
	gtk_widget_show(frame);

	table = gtk_table_new(3, 4, TRUE);
	gtk_container_set_border_width(GTK_CONTAINER (table), 10);
	gtk_table_set_row_spacings(GTK_TABLE (table), 4);
	gtk_table_set_col_spacing(GTK_TABLE (table), 0, 20);
	gtk_container_add(GTK_CONTAINER (frame), table);
	gtk_widget_show(table);

	lab_info_font = gtk_label_new("Informative Font: ");
	gtk_misc_set_alignment(GTK_MISC (lab_info_font), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_info_font), GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults(GTK_TABLE (table), lab_info_font, 0, 1, 0, 1);
	gtk_widget_show(lab_info_font);
	
	txt_info_font = gtk_entry_new_with_max_length(255);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "txt_info_font", txt_info_font);
	gtk_entry_set_text(GTK_ENTRY (txt_info_font), gde_rc->info_font);
	gtk_table_attach_defaults(GTK_TABLE (table), txt_info_font, 1, 4, 0, 1);
	gtk_widget_show(txt_info_font);

	lab_output_font = gtk_label_new("Output Font: ");
	gtk_misc_set_alignment(GTK_MISC (lab_output_font), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_output_font), GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults(GTK_TABLE (table), lab_output_font, 0, 1, 1, 2);
	gtk_widget_show(lab_output_font);
	
	txt_output_font = gtk_entry_new_with_max_length(255);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "txt_output_font", txt_output_font);
	gtk_entry_set_text(GTK_ENTRY (txt_output_font), gde_rc->output_font);
	gtk_table_attach_defaults(GTK_TABLE (table), txt_output_font, 1, 4, 1, 2);
	gtk_widget_show(txt_output_font);

	lab_error_font = gtk_label_new("Error Font: ");
	gtk_misc_set_alignment(GTK_MISC (lab_error_font), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_error_font), GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults(GTK_TABLE (table), lab_error_font, 0, 1, 2, 3);
	gtk_widget_show(lab_error_font);
	
	txt_error_font = gtk_entry_new_with_max_length(255);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "txt_error_font", txt_error_font);
	gtk_entry_set_text(GTK_ENTRY (txt_error_font), gde_rc->error_font);
	gtk_table_attach_defaults(GTK_TABLE (table), txt_error_font, 1, 4, 2, 3);
	gtk_widget_show(txt_error_font);

	/* GDE Preferences, General */
	hbox = gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show(hbox);
	
	lab_recent_size = gtk_label_new("Recent projects list size: ");
	gtk_misc_set_alignment(GTK_MISC (lab_recent_size), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL (lab_recent_size), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX (hbox), lab_recent_size, TRUE, TRUE, 0);
	gtk_widget_show(lab_recent_size);
	
	adj_recent_size = (GtkWidget *) gtk_adjustment_new(gde_rc->recent_size, 1, GDE_RECENT_MAX, 1, 1, 0);
	spin_recent_size = gtk_spin_button_new(GTK_ADJUSTMENT (adj_recent_size), 0, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON (spin_recent_size), TRUE);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "spin_recent_size", spin_recent_size);
	gtk_box_pack_start(GTK_BOX (hbox), spin_recent_size, FALSE, FALSE, 0);
	gtk_widget_show(spin_recent_size);

	opt_save_window_info = gtk_check_button_new_with_label("Save window positions/sizes on exit");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (opt_save_window_info), gde_rc->save_window_info);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "opt_save_window_info", opt_save_window_info);
	gtk_box_pack_start(GTK_BOX (vbox), opt_save_window_info, TRUE, TRUE, 0);
	gtk_widget_show(opt_save_window_info);
	
	opt_open_last_opened = gtk_check_button_new_with_label("Open last opened project on startup");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (opt_open_last_opened), gde_rc->open_last_opened);
	gtk_object_set_data(GTK_OBJECT (settings_dialog), "opt_open_last_opened", opt_open_last_opened);
	gtk_box_pack_start(GTK_BOX (vbox), opt_open_last_opened, TRUE, TRUE, 0);
	gtk_widget_show(opt_open_last_opened);
	
	/* The two buttons */
	gtk_container_set_border_width(GTK_CONTAINER (GTK_DIALOG (settings_dialog)->action_area), 2);
	gtk_box_set_homogeneous(GTK_BOX (GTK_DIALOG (settings_dialog)->action_area), FALSE);
	
	hbbox = gtk_hbutton_box_new();
	gtk_button_box_set_spacing(GTK_BUTTON_BOX (hbbox), 4);
	gtk_box_pack_end(GTK_BOX (GTK_DIALOG (settings_dialog)->action_area), hbbox, FALSE, FALSE, 0);
	gtk_widget_show(hbbox);

	cancel_button = gtk_button_new_with_label("Cancel");
	GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT (cancel_button), "clicked",
					   GTK_SIGNAL_FUNC(settings_win_cancel), NULL);
	gtk_box_pack_end(GTK_BOX (hbbox), cancel_button, FALSE, FALSE, 0);
	gtk_widget_show(cancel_button);

	ok_button = gtk_button_new_with_label("OK");
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
	gtk_box_pack_end(GTK_BOX (hbbox), ok_button, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT (ok_button), "clicked",
					   GTK_SIGNAL_FUNC(settings_win_ok), NULL);
	gtk_widget_grab_default(ok_button);
	gtk_widget_show(ok_button);

	/* Let the user do his things */
	gtk_notebook_set_page(GTK_NOTEBOOK (notebook), page);
	gtk_widget_show(settings_dialog);

	while (GTK_WIDGET_VISIBLE(settings_dialog))
		gtk_main_iteration_do(TRUE);

	gtk_widget_destroy(settings_dialog);
}


/*----------------------------------------------------------

	settings_win_project()

	Opens the settings win with the project tab.

----------------------------------------------------------*/
void settings_win_project(void)
{
	settings_win(0);
}


/*----------------------------------------------------------

	settings_win_preferences()

	Opens the settings window with the GDE preferences tab.

----------------------------------------------------------*/
void settings_win_preferences(void)
{
	settings_win(1);
}
