/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	November 30 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"


/*=========================================================

	Variable Declarations

=========================================================*/
static GtkWidget *status_win;
static GdeWindowInfo *status_win_info;

static GtkWidget *status_text;


/*---------------------------------------------------------

	status_win_wheelscroll()

	lets you scroll the status window with a mouse
	that has a wheel on it. We expect those to be
	buttons 4 and 5.
  
---------------------------------------------------------*/
void status_win_wheelscroll(GtkWidget* widget, GdkEventButton *bevent)
{
	gfloat value = GTK_TEXT (widget)->vadj->value;
	gfloat page_increment = GTK_TEXT (widget)->vadj->step_increment;

	if (bevent) {
			if (bevent->button == 5)
				value = value + page_increment;
			if (bevent->button == 4)
				value = value - page_increment;
	}
	
	gtk_adjustment_set_value(GTK_TEXT (widget)->vadj, value);
}


/*---------------------------------------------------------

	status_win_text_box_create()

	Creates the text box in the status window.
	
---------------------------------------------------------*/
static void status_win_text_box_create(void)
{
	GtkWidget *box;
	GtkWidget *vscrollbar;

	box = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER (status_win), box);
	gtk_widget_show(box);

	status_text = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT (status_text), FALSE);
	gtk_box_pack_start(GTK_BOX (box), status_text, TRUE, TRUE, 0);
	gtk_widget_show(status_text);

	gtk_signal_connect(GTK_OBJECT(status_text), "button_press_event", GTK_SIGNAL_FUNC(status_win_wheelscroll), NULL);
	
	vscrollbar = gtk_vscrollbar_new(GTK_TEXT (status_text)->vadj);
	gtk_box_pack_start(GTK_BOX (box), vscrollbar, FALSE, FALSE, 0);
	gtk_widget_show(vscrollbar);
}


/*---------------------------------------------------------

	status_win_create()

	Creates the status window, using GTK.

---------------------------------------------------------*/
void status_win_create(void)
{
	status_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	
	gtk_signal_connect(GTK_OBJECT (status_win), "delete_event",
					   GTK_SIGNAL_FUNC (status_win_hide),
					   NULL);
	gtk_window_set_title(GTK_WINDOW (status_win), "Messages");
	
	gtk_widget_set_uposition(status_win, STATUSWIN_LEFT, STATUSWIN_TOP);
	gtk_widget_set_usize(status_win, STATUSWIN_WIDTH, STATUSWIN_HEIGHT);
	gtk_window_set_policy(GTK_WINDOW (status_win), FALSE, TRUE, TRUE);
	gtk_container_set_border_width(GTK_CONTAINER (status_win), 2);
	gtk_window_add_accel_group(GTK_WINDOW (status_win), main_accel_group);

	status_win_text_box_create();
	
	status_win_info = g_malloc0(sizeof(GdeWindowInfo));
	gtk_widget_show(status_win);
}


/*----------------------------------------------------------

	status_win_destroy()

	Destroy the window and free all memory.

----------------------------------------------------------*/
void status_win_destroy(void)
{
	g_free(status_win_info);
	gtk_widget_destroy(status_win);
}


/*---------------------------------------------------------

	status_win_show()

	Pop the window on the screen.

---------------------------------------------------------*/
void status_win_show(void)
{
	if (!GTK_WIDGET_VISIBLE(status_win)) {
		status_win_set_window_info(status_win_info);
	}
}


/*---------------------------------------------------------

	status_win_hide()

	Remove the window from the screen.

---------------------------------------------------------*/
gint status_win_hide(void)
{
	if (GTK_WIDGET_VISIBLE(status_win)) {
		status_win_get_window_info(status_win_info);
		gtk_widget_hide(status_win);
	}
	return TRUE;
}


/*---------------------------------------------------------

	status_win_text_insert()

	Insert the string into the status window.

---------------------------------------------------------*/
void status_win_text_insert(const gchar *string, GdeStatusTextType status_text_type)
{
	GdkFont *font = NULL;

	GdkColormap *cmap;
	GdkColor color;
	
	cmap = gdk_colormap_get_system();
	if (status_text_type == STATUS_TEXT_INFO) {
		color.red = 0; color.green = 0;	color.blue = 0xFFFF;
		if (*gde_rc->info_font)
			font = gdk_font_load(gde_rc->info_font);
		else
			font = gdk_font_load("-adobe-helvetica-medium-r-normal-*-*-120-*-*-*-*-iso8859-1");
	}
	else if (status_text_type == STATUS_TEXT_OUTPUT) {
		color.red = 0; color.green = 0; color.blue = 0;
		if (*gde_rc->output_font)
			font = gdk_font_load(gde_rc->output_font);
		else
			font = gdk_font_load("-adobe-courier-medium-r-normal-*-*-120-*-*-*-*-iso8859-1");
	}
	else if (status_text_type == STATUS_TEXT_ERROR) {
		color.red = 0x8FFF; color.green = 0; color.blue = 0;
		if (*gde_rc->error_font)
			font = gdk_font_load(gde_rc->error_font);
		else
			font = gdk_font_load("-adobe-courier-medium-r-normal-*-*-120-*-*-*-*-iso8859-1");
	}
		
	if (!gdk_color_alloc(cmap, &color))
		g_error("Could not allocate color");

	gtk_text_insert(GTK_TEXT (status_text), font, &color, NULL, string, -1);
}


/*---------------------------------------------------------

	status_win_char_insert()

	Insert the character into the status window.

---------------------------------------------------------*/
void status_win_char_insert(gchar c, GdeStatusTextType status_text_type)
{
	gchar string[2];

	string[0] = c;
	string[1] = 0;
	
	status_win_text_insert(string, status_text_type);
}


/*---------------------------------------------------------

	status_win_clear()

	Clears the status window.

---------------------------------------------------------*/
void status_win_clear(void)
{
	gchar str[128];
	
	gtk_text_backward_delete(GTK_TEXT (status_text), 
		gtk_text_get_length(GTK_TEXT (status_text)));

	sprintf(str, GDE_WELCOME_MESSAGE, GDE_VERSION);
	status_win_text_insert(str, STATUS_TEXT_INFO);
}


/*----------------------------------------------------------

	status_win_get_window_info()

	Return the position, size and visibility state of 
	the window.

----------------------------------------------------------*/
void status_win_get_window_info(GdeWindowInfo *win_info)
{
	win_info->visible = GTK_WIDGET_VISIBLE(status_win);
	
	if (!win_info->visible) {
		win_info->x = status_win_info->x;	
		win_info->y = status_win_info->y;	
		win_info->width = status_win_info->width;	
		win_info->height = status_win_info->height;
	}
	else { 
		gdk_window_get_origin(GTK_WIDGET (status_win)->window, &win_info->x, &win_info->y);
		gdk_window_get_size(GTK_WIDGET (status_win)->window, &win_info->width, &win_info->height);
	}
}


/*----------------------------------------------------------

	status_win_set_window_info()

	Set the geometry and visibility of the window.

----------------------------------------------------------*/
void status_win_set_window_info(GdeWindowInfo *win_info)
{
	if (win_info->visible) {
		if (!GTK_WIDGET_VISIBLE(status_win))
			gtk_widget_show(status_win);

		gtk_widget_set_uposition(status_win, win_info->x, win_info->y);
		gdk_window_resize(status_win->window, win_info->width, win_info->height);
	}
	else {
		if (GTK_WIDGET_VISIBLE(status_win))
			status_win_hide();
		
		status_win_info->x = win_info->x;	/* Why this? I dunno,  seems like GTK has */
		status_win_info->y = win_info->y;	/* some bug in displaying  and hiding win */
		status_win_info->width = win_info->width;	/* Want to know why? Just comment */
		status_win_info->height = win_info->height; /* out these four  lines  of code */
		status_win_info->visible = TRUE;
	}
}
